@@ -1,6 +1,6 @@
 InfCr <- function(x,cr="AIC"){
 
-n <- x$n; edf <- x$t.edf; l <- x$fit$l   
+n <- x$n; edf <- x$t.edf; l <- x$logL   
 
 if(cr=="AIC")  value <- 2*l + 2*edf 
 #if(cr=="AICc") value <- 2*l + 2*edf + (2*edf*(edf+1))/(n-edf-1)  
